MODULE usrdef
   !!==============================================================================
   !!                       ***  MODULE usrdef   ***
   !! User defined module: used like example to define init, sbc, bathy, ...
   !!==============================================================================
   !! History :  NEMO ! 2016-03  (S. Flavoni) 
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   usr_def_hgr       : initialize the horizontal mesh 
   !!   usr_def_ini       : initial state 
   !!   usr_def_sbc       : initialize the surface bounday conditions
   !!----------------------------------------------------------------------
   USE step_oce       ! module used in the ocean time stepping module (step.F90)
   USE mppini         ! shared/distributed memory setting (mpp_init routine)             
   USE phycst         ! physical constants 
   IMPLICIT NONE
   PRIVATE


   PUBLIC usr_def_hgr
   PUBLIC usr_def_ini

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.7 , NEMO Consortium (2014)
   !! $Id:$ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE usr_def_hgr( nbench, jp_cfg, kff    , pff   , & ! Coriolis parameter  (if domain not on the sphere)
           &               pglamt, pglamu, pglamv , pglamf, & ! gridpoints position (required)
           &               pgphit, pgphiu, pgphiv , pgphif, & !         
           &               pe1t  , pe1u  , pe1v   , pe1f  , & ! scale factors (required)
           &               pe2t  , pe2u  , pe2v   , pe2f  , & !
           &               ke1e2u_v                       )   ! u- & v-surfaces (if gridsize reduction is used in strait(s))
      !!-------------------------------------------------------------------------------------------------
      !!                  ***  ROUTINE usr_def_hgr  ***
      !!
      !! ** Purpose :   user defined mesh and Coriolis parameter
      !!
      !! ** Method  :   set all intent(out) argument to a proper value
      !!
      !!                Here GYRE configuration :
      !!          Rectangular mid-latitude domain 
      !!          - with axes rotated by 45 degrees
      !!          - a constant horizontal resolution of 106 km 
      !!          - on a beta-plane
      !!
      !! ** Action  : - define longitude & latitude of t-, u-, v- and f-points (in degrees) 
      !!              - define coriolis parameter at f-point if the domain in not on the sphere (on beta-plane)
      !!              - define i- & j-scale factors at t-, u-, v- and f-points (in meters)
      !!              - define u- & v-surfaces (if gridsize reduction is used in some straits) (in m2)
      !!-------------------------------------------------------------------------------------------------

      !!----------------------------------------------------------------------
      INTEGER                 , INTENT(in   ) ::   nbench, jp_cfg   ! parameter of namelist for benchmark, and dimension of GYRE
      INTEGER                 , INTENT(  out) ::   kff              ! =1 Coriolis parameter computed here, =0 otherwise
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pff              ! Coriolis factor at f-point                  [1/s]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pglamt, pglamu, pglamv, pglamf !  longitude outputs        [degrees]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pgphit, pgphiu, pgphiv, pgphif !  latitude outputs         [degrees]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pe1t, pe1u, pe1v, pe1f   !  i-scale factors                      [m]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pe2t, pe2u, pe2v, pe2f   !  j-scale factors                      [m]
      INTEGER                 , INTENT(  out) ::   ke1e2u_v                 ! =1 u- & v-surfaces read here, =0 otherwise 
      !!-------------------------------------------------------------------------------
      INTEGER  ::   ji, jj               ! dummy loop indices
      REAL(wp) ::   zlam1, zlam0, zcos_alpha, zim1 , zjm1 , ze1  , ze1deg, zf0 ! local scalars
      REAL(wp) ::   zphi1, zphi0, zsin_alpha, zim05, zjm05, zbeta, znorme      ! local scalars
      !!-------------------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('usr_def_hgr')
      !
      !==  beta-plane with regular grid-spacing and rotated domain ==! (GYRE configuration)
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '          beta-plane with regular grid-spacing and rotated domain (GYRE configuration)'
      !
      ! angle 45deg and ze1=106.e+3 / jp_cfg forced -> zlam1 = -85deg, zphi1 = 29degN
      zlam1 = -85._wp
      zphi1 =  29._wp
      ! resolution in meters
      ze1 = 106000. / REAL( jp_cfg , wp )
      ! benchmark: forced the resolution to be about 100 km
      !
   END SUBROUTINE usr_def_hgr


   SUBROUTINE usr_def_zgr()           ! Coriolis parameter  (if domain not on the sphere)
       ! subroutine for vertical grid
   END SUBROUTINE usr_def_zgr

 
   SUBROUTINE usr_def_ini( pts )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE usr_def_ini  ***
      !! 
      !! ** Purpose :   Initialization of the dynamics and tracers
      !!                Here GYRE configuration example : (double gyre with rotated domain)
      !!
      !! ** Method  : - set temprature field
      !!              - set salinity   field
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(  out) ::   ptsd   ! T & S data
      !
      INTEGER :: ji, jj, jk  ! dummy loop indices
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'usr_def_ini : analytical definition of initial state '
      IF(lwp) WRITE(numout,*) '              T and S profiles deduced from LEVITUS '
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~'
      !
      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi
               pts(ji,jj,jk,jp_tem) =  (  (  16. - 12. * TANH( (gdept_n(ji,jj,jk) - 400) / 700 ) )   &
                    &           * (-TANH( (500-gdept_n(ji,jj,jk)) / 150 ) + 1) / 2               &
                    &           + ( 15. * ( 1. - TANH( (gdept_n(ji,jj,jk)-50.) / 1500.) )        &
                    &           - 1.4 * TANH((gdept_n(ji,jj,jk)-100.) / 100.)                    &
                    &           + 7.  * (1500. - gdept_n(ji,jj,jk)) / 1500.)                     &
                    &           * (-TANH( (gdept_n(ji,jj,jk) - 500) / 150) + 1) / 2  ) * tmask(ji,jj,jk)

               pts(ji,jj,jk,jp_sal) =  (  (  36.25 - 1.13 * TANH( (gdept_n(ji,jj,jk) - 305) / 460 ) )  &
                    &         * (-TANH((500 - gdept_n(ji,jj,jk)) / 150) + 1) / 2              &
                    &         + ( 35.55 + 1.25 * (5000. - gdept_n(ji,jj,jk)) / 5000.          &
                    &         - 1.62 * TANH( (gdept_n(ji,jj,jk) - 60.  ) / 650. )             &
                    &         + 0.2  * TANH( (gdept_n(ji,jj,jk) - 35.  ) / 100. )             &
                    &         + 0.2  * TANH( (gdept_n(ji,jj,jk) - 1000.) / 5000.) )           &
                    &         * (-TANH((gdept_n(ji,jj,jk) - 500) / 150) + 1) / 2  ) * tmask(ji,jj,jk)
            END DO
         END DO
      END DO
      !   
   END SUBROUTINE usr_def_ini

   !!======================================================================
END MODULE usrdef
